import React, {useMemo} from "react";
import {AutoColumnContextProvider, Column, ColumnHeading, ColumnKey, Columns, SortState, SortType} from "./Column";
import {SmartableProperties, useTable} from "./Smartable";
import {RowInstance, RowLoader} from "./Row";
import {CurrentRowData, useAsyncManager} from "./AsyncManager";

/**
 * Smartable instance component properties.
 */
export interface InstanceProperties<CK extends ColumnKey> extends SmartableProperties<CK>
{
	/**
	 * Table columns.
	 */
	columns: Columns<CK>;
}

/**
 * Main component for a Smartable table.
 */
export function Instance<CK extends ColumnKey>({columns}: InstanceProperties<CK>)
{
	return (
		<table className={"smartable"}>
			<thead>
			<ColumnsHeadings columns={columns} />
			</thead>
			<tbody>
			<TableBody />
			</tbody>
		</table>
	);
}

/**
 * Columns headings of a Smartable table.
 */
export function ColumnsHeadings<CK extends ColumnKey>({columns}: {columns: Columns<CK>})
{
	return (
		<>
			<tr className={"headings"}>
				{ // Showing title of each column.
					Object.keys(columns).map((key) => (
						<AutoColumnContextProvider key={key} columnKey={key}>
							<ColumnHeading/>
						</AutoColumnContextProvider>
					))
				}
			</tr>
			<tr className={"filters"}>
				{ // Add columns filters, if there are some.
					(Object.entries(columns) as [CK, Column][]).map(([columnKey, column]) => (
						column.filter && (
							<AutoColumnContextProvider key={columnKey as string} columnKey={columnKey}>
								<td>{column.filter.element}</td>
							</AutoColumnContextProvider>
						)
					))
				}
			</tr>
		</>
	);
}

export function TableBody<CK extends ColumnKey>()
{
	// Get data from table.
	const {data, columns, columnsSortState, columnsFilterStates} = useTable<CK>();

	// Get current data state from the async table value.
	const {currentDataState} = useAsyncManager<CK>(data);

	// Memorize filtered rows.
	const filteredRows = useMemo(() => (
		currentDataState.rows?.filter((row) => (
			// Checking each row to keep only those which match the filters.
			(Object.entries(columnsFilterStates) as [CK, any][]).every(([columnKey, filterState]) => (
				// For each filter, keep the row if data match the current filter.
				columns[columnKey].filter.filter(row.cells[columnKey].data, filterState)
			))
		))
	), [currentDataState.rows, columnsFilterStates]);

	// Memorize sorted rows.
	const sortedRows = useMemo(() => (
		sortRows<CK>(filteredRows, columns, columnsSortState)
	), [filteredRows, columns, columnsSortState]);

	return (
		sortedRows ? (
			sortedRows.map((rowData, index) => (
				// Rendering each row from its definition.
				<RowInstance row={rowData} />
			))
		) : (
			<RowLoader />
		)
	);
}

/**
 * Sort the given rows from the columns sort state.
 * @param rows Rows to sort.
 * @param columns Columns definition.
 * @param columnsSortState Columns current sort state.
 */
export function sortRows<CK extends ColumnKey>(rows: CurrentRowData<CK>[], columns: Columns<CK>, columnsSortState: Partial<Record<CK, SortState>>): CurrentRowData<CK>[]
{
	// Normalize value to undefined when rows are not loaded.
	if (!Array.isArray(rows)) return undefined;

	// Get the sort column states following their priority order.
	const orderedSortColumns = (Object.entries(columnsSortState) as [CK, SortState][]).sort((a, b) => (
		a[1].order - b[1].order
	));

	// Sort the given rows.
	return rows.toSorted((a, b) => {
		for (const [columnKey, columnSort] of orderedSortColumns)
		{
			// Get the current comparison result for the given cells' data.
			const comparison =
				(columnSort.type == SortType.DESC ? 1 : -1) *
				(columns[columnKey]?.compare ?? genericCompare)(a.cells[columnKey]?.data, b.cells[columnKey]?.data);

			if (comparison != 0)
				// If an order can be determined from the current comparison, returning it.
				return comparison;
		}

		// The comparisons always returned 0.
		return 0;
	});
}

/**
 * Generic comparison function for table data.
 * @param a First data to compare.
 * @param b Second data to compare.
 */
export function genericCompare(a: any, b: any): number
{
	if (typeof a == "number" && typeof a == typeof b)
	{ // Compare numbers.
		return b - a;
	}
	else
	{ // Compare data as strings.
		return String(b).localeCompare(String(a));
	}
}
